#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


class Error():
    ERROR_1010 = "Error:1010"
    ERROR_1011 = "Error:1011"

    ERROR_3010 = "Error:3010"

    ERROR_8010 = "Error:8010"
    ERROR_8011 = "Error:8011"
